@@ -5,7 +5,7 @@ from typing import Type
 
 from dbt.adapters.base.relation import BaseRelation
 from dbt.adapters.base.relation import Self
-from dbt.contracts.graph.parsed import ParsedSourceDefinition
+from dbt.contracts.graph.nodes import SourceDefinition
 
 
 @dataclass(frozen=True, eq=False, repr=False)
@@ -13,7 +13,7 @@ class DuckDBRelation(BaseRelation):
     external_location: Optional[str] = None
 
     @classmethod
-    def create_from_source(cls: Type[Self], source: ParsedSourceDefinition, **kwargs: Any) -> Self:
+    def create_from_source(cls: Type[Self], source: SourceDefinition, **kwargs: Any) -> Self:
 
         # Some special handling here to allow sources that are external files to be specified
         # via a `external_location` meta field. If the source's meta field is used, we include
